@@ -12,10 +12,12 @@ pub(crate) struct Node {
     pub(crate) weights: [i8; 4],
     pub depth: u8,
     start: char,
+    #[cfg(debug_assertions)]
+    pub phrase: &'static str,
 }
 
 impl Tree {
-    pub fn add(&mut self, word: &str, weights: [i8; 4]) {
+    pub fn add(&mut self, word: &'static str, weights: [i8; 4]) {
         let mut current = &mut self.root;
         for (i, c) in word.chars().enumerate() {
             let next = current.children.entry(c);
@@ -24,6 +26,8 @@ impl Tree {
                 weights: [0; 4],
                 depth: (i + 1) as u8,
                 start: word.chars().next().unwrap(),
+                #[cfg(debug_assertions)]
+                phrase: &word[0..=i],
             });
         }
         current.weights = weights;
@@ -38,6 +42,8 @@ impl FromIterator<(&'static str, [i8; 4])> for Tree {
                 weights: [0; 4],
                 depth: 0,
                 start: 0 as char,
+                #[cfg(debug_assertions)]
+                phrase: "",
             },
         };
         for (word, weights) in iter.into_iter() {
